#------------------------------------------------------------------------------
# Diagnostic script to get the inputs related to "CPU usage threshold
# crossing", "Memory usage threshold crossing" and  "Buffer usage
# threshold crossing" from the user
#
# October 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-10/23/2007
# Title: Store(configure) inputs from user and register with ERM and EEM.
# 
# Name: sdiag_resource.tcl
#
# Desc: Gets input from customer, validates them and and stores
#       (configures on the router) the inputs and registers the dynamically
#       created EEM policies using resource event detector, with EEM.
#
#       The inputs will be collected from user non-interactively
#
# Usage: tclsh disk#:/dir_name/sdiag_resource.tcl <resource name>
#        <notification> <configurationHistory> <EventHistory>
#        <userPolicyDirectory> <userLibraryDirectory> <process_name or
#        interrupt> <high_threshold> <low_threshold> <time_interval> 
#        <resource_allocation> 
#        where
#
#        The <resource name> --        ["cpu" or "memory" or "buffer"]
#            <notification> -- ["email" or "syslog" or "all" ] 
#            <configurationHistory> -- ["TRUE" or "FALSE"]
#            <EventHistory>  -- ["TRUE" or "FALSE"]
#            <userPolicyDirectory> -- [absolute path of policy directory]
#            <userLibraryDirectory> -- [absolute path of library directory]
#            <process_name or interrupt> -- [name of the process to be
#                                            monitored for threshold 
#                                            crossing or "intr" for
#                                            interrupt level CPU
#                                            utilization monitoring]
#            <high_threshold> -- [high level threshold in percentage]
#            <low_threshold> -- [low level threshold in percentage]
#            <time_interval> -- [the interval in seconds for which the
#                                monitoring is to be done]         
#            <resource_allocation> -- ["TRUE" or "FALSE"]
#        
#-----------------------------------------------------------------------------        
#############################################################################
#
#    Description: Creates an EEM policy on the fly based on the input
#                 given by the user 
#
#    IN: The url or the path where the policy has to be created and
#        saved    
#    OUT: NONE
#  
#############################################################################

proc createEEMPolicy {url name} {

    if {[catch {set fd [open $url w]} err] == 1 } {
             puts "\n\nCannot open file $url for writing!\n\n"
             return
    }   
    
    puts -nonewline $fd {::cisco::eem::event_register_resource policy sdiag-}
    puts -nonewline $fd $name
    puts $fd { maxrun 120
        
    namespace import ::cisco::eem::*
    namespace import ::cisco::lib::*

    array set arr_einfo [event_reqinfo]


    if {$_cerrno != 0} {
        set result [format \"component=%s; subsys err=%s; posix err=%s; \
                   \n%s\" $_cerr_sub_num $_cerr_sub_err $_cerr_posix_err \
                   $_cerr_str]
        error $result
    }


    set erm_configured_threshold $arr_einfo(configured_threshold)
    set erm_current_value $arr_einfo(current_value)
    set erm_user_id $arr_einfo(user_id)
    set erm_owner_id $arr_einfo(owner_id)


#############################################################################
#
#    Description: IOS super user command.
#                 This is for getting ios super command under enable 
#                 mode such as show command etc.
#
#    IN: ios cli command which needs to be executed. 
#    OUT: None.
#  
#############################################################################

    proc exec_show_commands {command} {
        if [catch {cli_open} result] {
            error $result $errorInfo
        } else {
            array set cli1 $result
        }

        if [catch {cli_exec $cli1(fd) "enable"} result] {
            error $result $errorInfo
        }

        if [catch {cli_exec $cli1(fd) $command} result] {
            error $result $errInfo
        } else {
            set show_command_output $result
        }

        if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
            error $result $errorInfo
        }
        return $show_command_output
    }

#############################################################################
#
#    Description: To get the name of the resource user 
#
#    IN: Resource User ID related to the event 
#    OUT: process name corresponding to the resource user id i.e.
#         resource user  
#  
#############################################################################

    proc getResourceUserDetails {userid} {
        
        set erm_process_name ""
        if {$userid != 0} {
            set output [exec_show_commands "show resource user all | i $userid"]
            for {set i 7} {$i < [llength $output]} {incr i} { 
                if {[regexp {#} [lindex $output $i]] != 1} { 
                    append erm_process_name "[lindex $output $i] "
                }
            }
        } else {
            set erm_process_name interrupt
        }

        return $erm_process_name
    }

#############################################################################
#
#    Description: IOS super user command.
#                 This is for getting ios super command under enable mode. 
#                 such as show command etc.
#
#    IN: ios cli command which needs to be executed. 
#    OUT: None.
#  
#############################################################################

    proc exec_show_commands {command} {
        if [catch {cli_open} result] {
            error $result $errorInfo
        } else {
            array set cli1 $result
        }
    
        if [catch {cli_exec $cli1(fd) "enable"} result] {
            error $result $errorInfo
        }
    
        if [catch {cli_exec $cli1(fd) $command} result] {
                error $result $errInfo
        } else {
            set show_command_output $result
        }
    
        if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
            error $result $errorInfo
        }
        return $show_command_output
    }

################################################################################
#
#    Description: This is to get the PID of process that crossed the threshold
#
#    IN: process name 
#    OUT: returns PID of the process
#  
#############################################################################

    proc getPID {name} {

        set output [exec_show_commands "sh proc cpu | i $name"]
        return [lindex $output 0]
    }

# check if all the environment variables we need exist
# If any of them doesn't exist, print out an error message and quit

    proc check_email_params {mode} {

        global _email_server _email_from _email_to _email_cc

        if {[string equal $mode "syslog"]} {
            return
        }

        if {![info exists _email_server]} {
            set result \
                "Policy cannot be run: variable _email_server has not been set"
            error $result "_email_server is required for sending email"
        }

        if {![info exists _email_from]} {
            set result \
                "Policy cannot be run: variable _email_from has not been set"
            error $result "_email_from is required for sending email"
        }
    
        if {![info exists _email_to]} {
            set result \
                "Policy cannot be run: variable _email_to has not been set"
            error $result "_email_to is required for sending email"
        }
    
        if {![info exists _email_cc]} {
            #_email_cc is an option, must set to empty string if not set.
            set _email_cc ""
        }
        return
    }

    if {![info exists _automation]} {
        set _automation FALSE
    }

    set messageString "\nTHE SERVICE DIAGNOSTIC MESSAGE FOR RESOURCE "
    set messageString_p2 ""
    set messageString_p3 ""
    set send_email 0
    set configHistory "\n------------------------------------------------\
    -------------------------------"
    append configHistory "\n\n THE CONFIGURATION HISTORY is: " 
    append configHistory [exec_show_commands "show archive config diff"]
    set eventHistory "\n------------------------------------------------\
    -------------------------------"
    append eventHistory \n\n " THE EVENT HISTORY is : "
    append eventHistory [exec_show_commands "show logging"] 

    set erm_process_name [getResourceUserDetails $erm_user_id]

    # Get Resource owner ID and the owner
    set OWNER ""
    set output [exec_show_commands "sh resource database"]
    foreach line [split $output "\n"] {
        if {[regexp {Owner: [a-zA-Z]+} $line] == 1} {
            set ownerId [split $line "Id:0x"]
            if {[lindex $ownerId 6] == $erm_owner_id} {
                set OWNER [lindex $ownerId 1]
            }
        }
    }
    if {[info exists erm_configured_threshold]} {
        set messageString_p3 " utilization threshold $erm_configured_threshold"
        if {[info exists erm_process_name]} {
            if {$erm_process_name == "interrupt"} {
               set messageString_p2 "\nInterrupt level CPU utilization \
                                      exceeded the configured "
            } else {
                set PID [getPID $erm_process_name]
                set messageString_p2 "\nProcess $erm_process_name with \
                                  PID $PID exceeded the configured  "
            }
        }
    } else {
        return
    }

    if {[regexp { memory                    } $OWNER] == 1} {
        if {[catch {check_email_params $_svcdiag_memNotification} err]} {
            error $err
        }
        append messageString MEMORY
        append messageString " MONITORING IS:\n"
        append messageString $messageString_p2
        append messageString MEMORY
        append messageString $messageString_p3
        if {$erm_current_value > $erm_configured_threshold} {
            if {($_svcdiag_memNotification == "syslog") ||
                ($_svcdiag_memNotification == "all")} {
                 action_syslog msg $messageString
            }
            if {($_svcdiag_memNotification == "email") ||
                ($_svcdiag_memNotification == "all")} {
                set send_email 1
                if {$_svcdiag_memConfigHistory == "TRUE"} {
                    append messageString $configHistory
                }
                if {$_svcdiag_memEventHistory == "TRUE"} {
                    append messageString $eventHistory
                }
                if {$_svcdiag_memAlloc == "TRUE"} {
                    append messageString "\n-----------------------------------------------------\
                    --------------------------"
                    append messageString \n\n "The MEMORY ALLOCATION AT THIS TIME is:\n" 
                    append messageString [exec_show_commands "show process memory "]
                }
            }
        }
    }

    if {[regexp { cpu                    } $OWNER] == 1} {
        if {[catch {check_email_params $_svcdiag_cpuNotification} err]} {
            error $err
        }
        append messageString CPU
        append messageString " MONITORING IS:\n"
        append messageString $messageString_p2
        append messageString CPU
        append messageString $messageString_p3
        if {$erm_current_value > $erm_configured_threshold} {
            if {($_svcdiag_cpuNotification == "syslog") ||
                ($_svcdiag_cpuNotification == "all")} {
                 action_syslog msg $messageString
            }
            if {($_svcdiag_cpuNotification == "email") ||
                ($_svcdiag_cpuNotification == "all")} {
                set send_email 1
                if {$_svcdiag_cpuConfigHistory == "TRUE"} {
                    append messageString $configHistory
                }
                if {$_svcdiag_cpuEventHistory == "TRUE"} {
                    append messageString $eventHistory
                }
            }
        }
    }

    if {[regexp { Buffer                    } $OWNER] == 1} {
        if {[catch {check_email_params $_svcdiag_bufNotification} err]} {
            error $err
        }
        append messageString BUFFER
        append messageString " MONITORING IS:\n"
        append messageString $messageString_p2
        append messageString  BUFFER
        append messageString $messageString_p3
        if {$erm_current_value > $erm_configured_threshold} {
            if {($_svcdiag_bufNotification == "syslog") ||
                ($_svcdiag_bufNotification == "all")} {
                 action_syslog msg $messageString
            }
            if {($_svcdiag_bufNotification == "email") ||
                ($_svcdiag_bufNotification == "all")} {
                set send_email 1
                if {$_svcdiag_bufConfigHistory == "TRUE"} {
                    append messageString $configHistory
                }
                if {$_svcdiag_bufEventHistory == "TRUE"} {
                    append messageString $eventHistory
                }
                if {$_svcdiag_bufAlloc == "TRUE"} {
                    append messageString "\n-----------------------------------------------------\
                    --------------------------"
                    append messageString \n\n "The BUFFER ALLOCATION AT THIS TIME is:\n" 
                    append messageString [exec_show_commands "show buffers"]
                }
            }
        }
    }


    set user_lib_dir [exec_show_commands "show event man dir user lib"]
    if {[regexp "not configured" $user_lib_dir match]} {
        error "User library directory is not configured"
    }
    set user_lib_dir [lindex $user_lib_dir 0]
    
    # send the email out
    
    if {$send_email == "1"} {
        set routername [info hostname]
        if {[string match "" $routername]} {
            error "Host name is not configured"
        }
    
        if [catch {smtp_subst [file join $user_lib_dir email_template]} result] {
            error $result $errorInfo
        }
    
        if [catch {smtp_send_email $result} result] {
            error $result $errorInfo
        } else {
            if {$_automation == "TRUE"} {
                puts $messageString
            }
        }
    }
    }
    close $fd
}

#############################################################################
#
#    Description: Checks if the user library directory is configured and 
#                 registered with EEM
#
#    IN :None
#    OUT:sources the files userlib.tcl and diag_lib.tcl 
#
#############################################################################    
proc sourceLibFile {user_lib} {
    
    if {[file exists [file join $user_lib diag_lib.tcl]]} {
        set user_lib_dir $user_lib
    } else {
        set user_lib_dir [exec show event man dir user lib]
        if {[regexp "not configured" $user_lib_dir match]} {
            puts "SERVICE DIAGNOSTICS ERROR: User library directory is \
            not configured nor does specified path exist."
            return 0
        }
    }
    
    source [file join $user_lib_dir diag_lib.tcl]
    source [file join $user_lib_dir userlib.tcl]
    return 1

}

#############################################################################
#
#    Description: Registers the user policy directory and user library
#                 directory with EEM
#
#    IN: "type" which is either "policy" or "library" and "dir_name"
#        is the name of the policy directory or name of the library
#        directory 
#    OUT: Gives error message whether the registration of user policy
#         directory or user library directory was successful or not.   
#
##############################################################################                             
proc resource_set_eem_dir { type dir_name } {
    
    if {[file exists $dir_name]}  {
         if {[catch {ios_config "event manager directory user $type
                 $dir_name"} \
             err]} {
            set err [string trim $err]
            puts "err is \"$err\" --- Failed to config EEM user $type directory"
            return 0
        } else {
            return 1
        }
    }
    return 0
}

#############################################################################
#
#    Description: Validates the process name input given by the user
#
#    IN: A string which contains the process name    
#    OUT: returns 0 if the input is a valid process name otherwise
#         returns 1 
#
##############################################################################
    
proc checkProcessName {value} {

    set proc_data [split [exec show proc cpu | i $value] "\n"]

    if {$proc_data == ""} {
        return 1
    } else {
        return 0
    }
}

#############################################################################
#
#    Description:  Validates the inputs specific to ERM scenarios 
#
#    IN: Process name, High and Low threshold values  
#    OUT: returns error message if valid inputs are not provided otherwise
#         returns from the procedure after complete execution of the
#         procedure 
#   
###############################################################################

proc validateERMInputs {ht lt emm} {

    set error 0

    if {([checkRange $ht] == 1) || ([checkRange $lt] == 1)} {
        puts "The high threshold value you entered is $ht"
        puts "The low threshold value you entered is $lt"
        puts "You should enter value between 1 and 99\n"
        set error 1
    } else {
        if {$ht < $lt} {
            puts "The high threshold value you entered is $ht"
            puts "The low threshold value you entered is $lt"
            puts "The low threshold value is greater than or equal to the \
                  high threshold value\n"
            set error 1
        }
    }

    return $error

}
  
###############################################################################
#
#    Description: If process name has a space inbetween this procedure 
#                 adds a string to replace the space. Since the EEM 
#                 policy name should be a continous string without space 
#                 we use this procedure  
#
#    IN: Process name entered by the user 
#    OUT: None.
#
###############################################################################

proc getProcessName {name} {

    if {[string equal -nocase $name "intr"]} {
        return "interrupt"
    }

    set output [split $name " "]
    set len [llength $output]
    
    if {$len < 2} {
        return $name
    } else {
        for {set i 0} {$i < [expr $len - 1] } {incr i} {
            append pname "[lindex $output $i]-xyz-"
        }
    }
    append pname [lindex $output $i]
    return $pname
}

################################################################################
#
#    Description: Checks if ios_config command was successful, if not 
#                 returns error
#                 
#    IN: args will be the config command 
#    OUT:Gives error message if the configuration on the router was
#        successful otherwise prints nothing.   
#       
###############################################################################  
proc ios_conf {args} {

    set config_list [list ios_config]
    foreach arg $args {
        lappend config_list $arg
    }

    if {[catch {[eval $config_list]} err]} {
        set err [string trim $err]
        puts "err is \"$err\" --- Failed to configure the \
              command \"$config_list\""    
        return 1
    }

}
                                            
#############################################################################
#
#    Description: If the notification is email then we check if the
#                 email related inputs have been given by the user
#                 previously, otherwise an error message is sent to the
#                 user to give the inputs related to email
#
#    IN: Notification which is "email" or "all"    
#    OUT: Returns 1 if the email inputs were not previously given
#         otherwise returns 0
#
##############################################################################

proc configure_email {notif} {

    set err 0
    
    if {($notif == "email") || ($notif == "all")} {    
        if {[emailInputAlreadyGiven  $notif] != "configured"} {
            puts "\nNo inputs corresponding to email was collected"
            puts "Run the file 'collectEmailParameters.tcl'"
            puts "Usage: tclsh disk#:/dir_name/collectEmailParameters.tcl \ 
                email_fromID email_toID email_ccID email_server"        
            set err 1
        }   
    }    
    
    return $err
    
}

#############################################################################
#
#    Description: Validates the input given by the user for the Resource
#                 Allocation option 
#
#    IN: The TRUE or FALSE value input by the user     
#    OUT: returns 1 if the value entered is not TRUE or FALSE otherwise 
#         returns 0
#  
#############################################################################

proc validateResAllocFlag {value} {

    if {[checkFlag $value] == 1} {
        puts "The value you entered for resourceAllocation is $value"
        puts "Enter either 'TRUE' or 'FALSE'"
        return 1
    }
}

#############################################################################
#
#    Description: Configure EEM environment variables for process level
#                 or interrupt level cpu threshold monitoring 
#
#    IN: Notification which "email" or "syslog" or "all", Configuration
#        and Event history option which is either "TRUE" or "FALSE",
#        name which is either a valid process name or the value
#        "interrupt", ht is the high threshold percentage, lt is the
#        low threshold percentage and time is the time interval
#        (in seconds) for which the threshold crossover is monitored. 
#    OUT: Configures the inputs as environment variables on the router 
#      
###############################################################################

proc cpu_config {notif config event name ht lt time res_alloc pname} {

    if {$name == "interrupt"} {
        if {[ios_conf "resource policy" "policy sdiag-interrupt global" \
            "system" "cpu interrupt" "minor rising $ht interval $time \
            falling $lt interval $time" "exit" "exit" "exit" "user global \
            sdiag-interrupt" "exit"] == "1"} {
            return 1
        }    
    } else {
        if {[ios_conf "resource policy" "policy sdiag-$name type \
            iosprocess" "system" "cpu process" "minor rising $ht interval \
            $time falling $lt interval $time" "exit" "exit" "exit" \
            "user \"$pname\" iosprocess sdiag-$name" "exit"] == "1"} {
            return 1
        }    
    }                    

    set_eem_env _svcdiag_cpuNotification $notif
    set_eem_env _svcdiag_cpuConfigHistory $config
    set_eem_env _svcdiag_cpuEventHistory $event
    
}

#############################################################################
#
#    Description: Configure EEM environment variables for process level
#                 buffer threshold monitoring 
#
#    IN: Notification which "email" or "syslog" or "all", Configuration
#        and Event history option which is either "TRUE" or "FALSE",
#        name which is either a valid process name or the value
#        "interrupt", ht is the high threshold percentage, lt is the
#        low threshold percentage and time is the time interval(in
#        seconds) for which the threshold crossover is monitored.  
#    OUT: Configures the inputs as environment variables on the router 
#      
################################################################################

proc buffer_config {notif config event name ht lt time res_alloc pname} {
   
    if {[ios_conf "resource policy" "policy sdiag-$name type \
        iosprocess" "system" "buffer public" "minor rising $ht interval \
        $time falling $lt interval $time" "exit" "exit" "exit" "user \
        \"$pname\" iosprocess sdiag-$name" "exit"] == "1"} {
        return 1    
    }
    
    set_eem_env _svcdiag_bufNotification $notif
    set_eem_env _svcdiag_bufConfigHistory $config
    set_eem_env _svcdiag_bufEventHistory $event
    set_eem_env _svcdiag_bufAlloc $res_alloc
    
}

#############################################################################
#
#    Description: Configure EEM environment variables for process level
#                 memory threshold monitoring
#
#    IN: Notification which "email" or "syslog" or "all", Configuration 
#        and Event history option which is either "TRUE" or "FALSE", 
#        name which is either a valid process name or the value
#        "interrupt", ht is the high threshold percentage, lt is the
#        low threshold percentage and time is the time interval(in
#        seconds) for which the threshold crossover is monitored. 
#    OUT: Configures the inputs as environment variables on the router
#
################################################################################                 

proc memory_config {notif config event name ht lt time res_alloc pname} {

    if {[ios_conf "resource policy" "policy sdiag-$name type \
        iosprocess" "system" "memory process" "minor rising $ht interval \
        $time falling $lt interval $time" "exit" "exit" "exit" "user \
        \"$pname\" iosprocess sdiag-$name" "exit"] == "1"} {
        return 1
    }

    set_eem_env _svcdiag_memNotification $notif
    set_eem_env _svcdiag_memConfigHistory $config
    set_eem_env _svcdiag_memEventHistory $event
    set_eem_env _svcdiag_memAlloc $res_alloc
    
}

###############################################################################
#
#    Description: This procedure sends a warning message to user if
#                 the user wants the output of configuration history and
#                 event history to be sent along with the syslog message
#
#    IN : Notification, configuration history option and event history
#         option
#    OUT: Sends a warning message that the configuration history
#         or event history will not be sent along with syslog message
#         if notification is "syslog" or "all"
#         
###############################################################################         
proc send_sdiag_warning {notif config event res_alloc} {

    if {($notif == "syslog") || ($notif == "all")} {
        if {($config == "TRUE") || ($event == "TRUE") || \
            ($res_alloc == "TRUE")} {
            puts "\n\nSERVICE-DIAGNOSTICS MESSAGE:"
            puts "\nAdditional diagnostic information like configuration \
                 history or event history or resource allocation cannot \
                 be sent along with syslog notification\n"
        }
    }
}

    # Execution of the script starts from here
    # Check if the number of arguments entered by user is correct
    # otherwise send error message. 
              
set error "\nFOR CPU THRESHOLD CROSSOVER MONITORING"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_resource.tcl \
             <cpu> <notification> \
             <configurationHistory> <EventHistory> <user_pol_dir> \
             <user_lib_dir> <process_name or interrupt> <high_threshold> \
             <low_threshold> <time_interval>"
append error "\n"
append error "\nFOR BUFFER THRESHOLD CROSSOVER MONITORING"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_resource.tcl \
             <buffer> <notification> \
             <configurationHistory> <EventHistory> <user_pol_dir> \
             <user_lib_dir> <process_name> <high_threshold> \
             <low_threshold> <time_interval> <resource_allocation>"
append error "\n"
append error "\nFOR MEMORY THRESHOLD CROSSOVER MONITORING"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_resource.tcl \
             <memory> <notification> \
             <configurationHistory> <EventHistory> <user_pol_dir> \
             <user_lib_dir> <process_name> <high_threshold> \
             <low_threshold> <time_interval> <resource_allocation>"


if {($argc == 10) || ($argc == 11)} {
    set res_name [lindex $argv 0]
    set notif [lindex $argv 1]
    set config [lindex $argv 2]
    set event [lindex $argv 3]
    set user_pol_dir [lindex $argv 4]
    set user_lib_dir [lindex $argv 5]
    set pname [lindex $argv 6]
    set high_threshold [lindex $argv 7]
    set low_threshold [lindex $argv 8]
    set time_interval [lindex $argv 9]

    if {[sourceLibFile $user_lib_dir] == 0} {
        return
    }

    if {$argc == 11} {
        set res_alloc [lindex $argv 10]
        set validation0 [validateResAllocFlag $res_alloc]
    } else {
        set res_alloc FALSE
        set validation0 0
    }

    set validation1 [validateERMInputs $high_threshold \
            $low_threshold $tcl_interactive]
    set validation2 [validateOtherInputs $notif $config $event]
    set validateERMTimer [checkIntervalLimit $time_interval]

    if {($validation0 == "1") || ($validation1 == "1") || \
        ($validation2 == "1") || ($validateERMTimer == "1")} {
        return
    }

    if {($pname != "intr") && ([checkProcessName $pname] == 1)\
        && ($tcl_interactive != "1")} {
        puts "\n\nSERVICE DIAGNOSTICS WARNING MESSAGE:\n"
        puts "$pname is not currently running or is invalid. \
            Still the registration is done.  You may or may not get threshold\
            crossover alert.\n"
        puts "If you want to monitor Interrupt Level CPU utilization, \
            enter 'intr' as input and redeploy the script.\n"
    }
            
    set arg_num_chk 0

    switch $res_name {

       cpu {
           if {$argc != 10} {
               set arg_num_chk 1
           } else {
               set erm_config cpu_config
           }
       }

       memory {
           if {$argc != 11} {
               set arg_num_chk 1
           } else {
               set erm_config memory_config
           }
       }

       buffer {
           if {$argc != 11} {
               set arg_num_chk 1
           } else {
               set erm_config buffer_config
           }
        }
       
        default {
            puts "\n\nSERVICE DIAGNOSTICS ERROR : Invalid resource name - \
            $res_name $error"
            return
        }     
    }

    if {$arg_num_chk == 1} {
        puts "\n\nSERVICE DIAGNOSTICS ERROR: Incorrect number of arguments"
        puts $error
        return
    }

    send_sdiag_warning $notif $config $event $res_alloc

    #
    # If process name has a space inbetween substitute it with 
    # some string xyz
    #

    set name [getProcessName $pname]

    if {[resource_set_eem_dir policy $user_pol_dir] == 0} {
        puts "\n\nSERVICE DIAGNOSTICS ERROR: Cannot read user policy directory."
        return
    }

    if {[resource_set_eem_dir library $user_lib_dir] == 0} {
        return
    }
    
    # Get email inputs and configure if they don't exist  
        
    if {[configure_email $notif] == 1} {
       return 
    }

    # Call the procedure to do the erm and eem configuration    
          
    if {[$erm_config $notif $config $event $name $high_threshold \
        $low_threshold $time_interval $res_alloc $pname] == "1"} {
         puts "ERM policy configuration failed" 
         return
    }

    # Procedure Call to create dynamic EEM policy and registering it with
    # EEM

    createEEMPolicy [file join $user_pol_dir sdiag-$name.tcl] $name
    unreg_eem_policy sdiag-$name.tcl
    reg_eem_policy sdiag-$name.tcl
    
} else {
    puts "\n\nSERVICE DIAGNOSTICS ERROR: Incorrect number of arguments"
    puts $error
    return
}

#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3038313032373233303332365a302306092a864886f70d010904
#31160414048501f2afc985142956370393f2a6dd01744efd305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#0105000482010031f93bb9f3a6465c854f85ea1edbb907fe31a1e9693f6b
#8ecae7eddd1e54f88e62ab235cb14f1c4ff93967ae03e052f697907fb1d4
#978d09ba85a3411703cb569dd235eb885e8ede5c5cec11310f7eed8ef5b3
#4d1b76a130986f27b70d0b886f921ab0c47516d3729f763a8fb1f1240226
#21ffe2ebc0f9940b67ff4b4c7ba8f3ae52dff34b3963449ff3d37bb85731
#d47cf6f749077ed8a8386544e553290147c6bf2a2f14917ee171786aca94
#4611a03a185df90ff8bafacd64251f11499734ba59cbf6001906bcdf578b
#5a081258c7f725dd1801dacd28e835eeb9a5a5e244759cc270579b420061
#705fef83862c0d9bc740813ae426247bf6c5ddb3d79b3f
