
# Sourcing the library file
 
set user_lib_dir [exec show event man dir user lib]
if {[regexp "not configured" $user_lib_dir match]} {
error "User library directory is not configured"
}

source [file join $user_lib_dir userlib.tcl]

#---------------------------------------------------------------------------
# Diagnostic script to display the resource inputs configured on the
#            router  
#
# November 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-11/05/2007 
# Title: Display the inputs related to resources 
#
# Name:  sdiag_resource_display.tcl
#
# Desc: Displays the details related to resources passed as arguments.
#       These are the inputs given by the user to monitor the CPU or
#       Memory or Buffer threshold crossing, for processes.  
#    
# Usage: tclsh disk#:/dir_name/sdiag_resource_display.tcl [resource name]
#        where
#        [resource name] -- [cpu] or [memory] or [buffer]
#
#---------------------------------------------------------------------------

global type process
set process {}

#############################################################################
#
#    Description: Executes "show event manager environment command and
#                 parses the output for configuration corresponding to the
#                 inputs entered by the user and displays them in a format
#                 easy for the user to understand  
#
#    IN: NONE   
#    OUT: Stores the parsed result in lists and calls the procedure 
#         "displayEnvVariables" 
#  
#############################################################################

proc showEEMVariables {res_type} {

### Initialize list which will store the configured inputs ###

    set cpuVar {}
    set bufVar {}
    set memVar {}

    set cpuOption "FALSE"
    set bufOption "FALSE"
    set memOption "FALSE"

    switch -regexp -- $res_type {

        cpu {
            set cpuOption "TRUE"
        }
        memory {
            set memOption "TRUE"
        }
        buffer {
            set bufOption "TRUE"
        }
        default {
            puts "No match"
        }
    }

    if {[catch {exec "show event manager environment"} result] != 0} {
        puts stderr "'show event manager environment' did not execute"
    } else {
        set output [split $result "\n"]

        foreach line $output {
            set envVar [lindex $line 1]
            set value [lindex $line 2]

            switch -regexp -- $envVar {
                cpu {
                        lappend cpuVar $envVar:$value
                }
                mem {
                        lappend memVar $envVar:$value
                }
                buf {
                        lappend bufVar $envVar:$value
                }
                default {
                }
            }
        }

        if {$cpuOption == "TRUE"} {
            if {[llength $cpuVar] != 0} {
                displayEnvVariables $cpuVar "CPU"
            } else {
                puts "NO INPUTS WERE GIVEN FOR CPU\n"
                return 1
            }
        }

        if {$memOption == "TRUE"} {
            if {[llength $memVar] != 0} {
                displayEnvVariables $memVar "MEMORY"
            } else {
                puts "NO INPUTS WERE GIVEN FOR MEMORY\n"
                return 1
            }
        }

        if {$bufOption == "TRUE"} {
            if {[llength $bufVar] != 0} {
                displayEnvVariables $bufVar "BUFFER"
            } else {
                puts "NO INPUTS WERE GIVEN FOR BUFFER\n"
                return 1
            }
        }
    }
}

#############################################################################
#
#    Description: Separates the process name from the policy name provided
#                 as input and returns the process name 
#
#    IN: ERM policy name 
#    OUT: Returns the process name  
#
#############################################################################

proc proc_name {name} {

    foreach i $name {
        if {$i != "sdiag"} {
            lappend pol_name $i
        }
    }
    foreach j $pol_name {
        if {$j != "xyz"} {
            lappend procList $j
        }
    }
    return $procList

}

#############################################################################
#
#    Description: Executes "show event manager policy registered" command 
#                 parses the output and displays all the EEM policies
#                 related to resources 
#
#    IN: None 
#    OUT: Displays all the registered EEM resource policies   
#  
#############################################################################

proc displayRegisteredEEMPolicies {procList res_type} {
   
    if {[catch {exec "show event manager policy registered"} result] != 0} {
        puts stderr "'show event manager policy registered' did not execute"
    } else {
        if {$result != ""} {
            set output [split $result "\n"]
            foreach line $output {
                if {[llength $line] == 11} {
                    lappend policyList [lindex $line 10]
                }
            }
        }
        if {[info exists policyList] && [llength $policyList] != 0} { 
            foreach policy $policyList {
                if {[regexp {sdiag-} $policy a] == 1} {
                    lappend pol $policy        
                } else {
                    if {[regexp {resource_} $policy] == 1} {
                        lappend action_pol $policy        
                    }
                }
            }
        }    
        if {[llength $procList] != 0} { 
            puts "\nTHE REGISTERED POLICIES ARE:"
            foreach i $procList {
                foreach p $pol {
                    set name [split $p "-"]
                    set pname [proc_name $name]  
                    set pname [split $pname "."]
                    if {[lindex $pname 0] == $i} {
                        puts $p        
                    }
                }
            }
        }    
        if {[info exists action_pol] && [llength $action_pol] != 0} {
            foreach j $action_pol {
                if {$j == $res_type} {
                    puts $j
                }
            }
        }
    }
}

#############################################################################
#
#    Description: Parses the router for ERM related inputs and processes it
#                 and displays on the router  
#
#    IN: NONE   
#    OUT:  Displays the configured ERM inputs on the router.
#  
#############################################################################

proc displayERMVariables {pol_type} {

    set process {}
    set pol_type [split $pol_type "_"]
    set res_type [lindex $pol_type 1]

# get the resource policy configuration section
    set output [exec "sh run | section i resource policy"]
    foreach line [split $output "\n"] {
        if {[regexp {policy sdiag-*} $line a] == 1} {
            set policy [split $line " "]
            lappend policyName [lindex $policy 3]
        }
    }
# get each ERM policy section and its details    
    if {[info exists policyName] && [llength $policyName] != 0} {
        foreach index $policyName {
            if {[regexp {interrupt} $index a] == 1} {
                set processDetails [exec "sh run | section i policy $index\
                    global"]        
            } else {
                set processDetails [exec "sh run | section i policy $index\
                    type iosprocess"]
            }
            set res [split $index "-"]
            set procname [proc_name $res]
            foreach line [split $processDetails "\n"] {
                lappend result $line
            }
            
# Based on the configuration of each resource for a single ERM policy
# inputs are parsed and displayed

            switch [llength $result] {
                6 -
                7 {
        ### when one resource is configured ###      
                      set values [split [lindex $result 3] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 2] " "]
                      set resource [lindex $out 4]
                      set pol [split [lindex $result 6] " "]
                      if {($procname == "interrupt") && ($resource == \
                          $res_type)} {
                          lappend process "interrupt"    
                          puts "\n\nInterrupt Level CPU, High-threshold:\
                              $highthreshold%, Low-threshold: $lowthreshold%,\
                              time-interval: $timeinterval sec"
                      } else {
                          if {$resource == $res_type} {
                              lappend process $procname
                              puts "\n\nProcess: $procname, Type: $resource,\
                                  High-threshold: $highthreshold%,\
                                  Low-threshold: $lowthreshold%,\
                                  time-interval: $timeinterval sec"
                          }
                      }
                  }
                9 -
                10 {
        ### when 2 resources are configured ###    
                      set values [split [lindex $result 3] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 2] " "]
                      set resource [lindex $out 4]
                      set pol [split [lindex $result 9] " "]
                      if {$resource == $res_type} {
                          lappend process $procname
                          puts "\n\nProcess: $procname, Type: $resource,\
                              High-threshold: $highthreshold%, \
                              Low-threshold: $lowthreshold%, \
                              time-interval: $timeinterval sec"
                      }
                      set values [split [lindex $result 6] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 5] " "]
                      set resource [lindex $out 4]
                      if {$resource == $res_type} {
                          lappend process $procname
                          puts "\n\nProcess: $procname, Type: $resource,\
                              High-threshold: $highthreshold%, Low-threshold:\
                              $lowthreshold%, time-interval: $timeinterval sec"
                      }
                 }
               13 {
        ### when 3 resources are configured ###
                      set values [split [lindex $result 3] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 2] " "]
                      set resource [lindex $out 4]
                      set pol [split [lindex $result 12] " "]
                      if {$resource == $res_type} {
                          lappend process $procname
                          puts "\n\nProcess: $procname, Type: $resource, \
                              High-threshold: $highthreshold%, \
                              Low-threshold: $lowthreshold, \
                              time-interval: $timeinterval sec"
                      }
                      set values [split [lindex $result 6] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 5] " "]
                      set resource [lindex $out 4]
                      if {$resource == $res_type} {
                          lappend process $procname
                          puts "\n\nProcess: $procname, Type: $resource, \
                              High-threshold: $highthreshold, \
                              Low-threshold: $lowthreshold, \
                              time-interval: $timeinterval sec"
                      }
                      set values [split [lindex $result 9] " "]
                      set highthreshold [lindex $values 7]
                      set timeinterval [lindex $values 9]
                      set lowthreshold [lindex $values 11]
                      set out [split [lindex $result 8] " "]
                      set resource [lindex $out 4]
                      if {$resource == $res_type} {
                          lappend process $procname
                          puts "\n\nProcess: $procname, Type: $resource, \
                              High-threshold: $highthreshold, \
                              Low-threshold: $lowthreshold, \
                              time-interval: $timeinterval sec"
                      }
                   }
            default {
                      puts "NO CONFIGURATION"
              }
            }
            set result {}
        }
    }
    return $process
}

### Script execution starts here ###

set error "\nUsage: tclsh disk#:/dir_name/sdiag_resource_display.tcl \
          <cpu>"
append error "\n\t\tOR"
append error "\nUsage: tclsh disk#:/dir_name/sdiag_resource_display.tcl \
             <memory>"
append error "\n\t\tOR"
append error "\nUsage: tclsh disk#:/dir_name/sdiag_resource_display.tcl \
             <buffer>"

if {$argc == 0} {
    global type
    if {$tcl_interactive == "1"} {
    } else {
        puts "\n\nSERVICE-DIAGNOSTICS ERROR: No arguments were given"
        puts $error
        return
    }
} else {
    if {$argc != 1} {
        puts "\n\nSERVICE-DIAGNOSTICS ERROR: No correct number of arguments"
        puts $error
        return
    }
}

# to take care of non-interactive inputs ###

if {$argc == 1} {
    set type [lindex $argv 0]
### check validity of non-interactive inputs and give error message ###

    set msg "\n\nSERVICE-DIAGNOSTICS ERROR: Input not valid"
    append msg "\nThe value you entered is: "
    
    if {($type != "cpu") && ($type != \
        "memory") && ($type !=\
        "buffer")} {
        puts "$msg $type"
        puts "\nEnter either <cpu> or <memory> or <buffer>"
        return
    }
}

# call procedure "showEEMVariables" to display scenario specific details
# based on argument passed

set procList [displayERMVariables $type] 
displayRegisteredEEMPolicies $procList $type
showEEMVariables $type 


#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3038303430373230323031375a302306092a864886f70d010904
#31160414ded736cc7513291bfc22f37efba1ca46efdb92fa305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#01050004820100005c5d05cda233bff0df47857a3bdf1c7635fb9aed2671
#83faf5fa0383e798676911ac944b1b7a9dfa558438f3cac63c23f5792fe6
#b90cee597a7a1ce8bab7a37df355438e99276799bd6d3c1a65031c50f921
#1eef25621026549cb02792fbfecb1bac8ec381fb4f39697dcf48704386c8
#f963238c2748bf29ed5d348a47de05db155f5b1052be87086f33ed876f55
#c0b6a6654682df0f086658467d56b969a731fdfcd9defb96cd6a4cff80f9
#7c042ca907396eebc2d3d2447fb4b991ac0e2eadbed2f001dee9a77bb5e8
#c133754d2c6c80ad8c5275b1002844a052aa95d0214ef071cb6fa89f14bd
#b9ce1887ea6c4e436a67a9328c2cc950298ec0a9a70c00
