
set user_lib_dir [exec show event man dir user lib]
if {[regexp "not configured" $user_lib_dir match]} {
    error "User library directory is not configured"
}

# Sourcing the library file
source [file join $user_lib_dir userlib.tcl]
source [file join $user_lib_dir diag_lib.tcl]


#-----------------------------------------------------------------------------
# Diagnostic script to undeploy the existing inputs and policies 
#
# July 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-07/20/2007 
# Title: Remove the inputs, related to ERM policies
# 
# Name:  sdiag_resource_undeploy.tcl
#
# Desc: Undeploys i.e. removes all the inputs related to the resource
#       scenarios passed as parameter. The inputs can be
#       collected from user non-interactively
#
# Usage: tclsh disk#:/dir_name/sdiag_resource_undeploy.tcl <resource name> 
#              <process name>                 
#        
#       The <resource name> -- "cpu" or "memory" or "buffer"
#           <process name> -- A process name or "interrupt"  
#
#-----------------------------------------------------------------------------

global error type show_run_output

##############################################################################
#
#    Description: If process name has a space inbetween this procedure 
#                 adds a string to replace the space. Since the EEM 
#                 policy name should be a continous string without space 
#                 we use this procedure  
#
#    IN: Process name entered by the user 
#    OUT: Returns process name with special characters if there is a 
#         space inbetween the name otherwise returns the name 
#
##############################################################################

proc getProcessName {name} {

    set output [split $name " "]
    set len [llength $output]

    if {$len < 2} {
        return $name
    } else {
        for {set i 0} {$i < [expr $len - 1] } {incr i} {
            append pname "[lindex $output $i]-xyz-"
        }
    }
    append pname [lindex $output $i]
    return $pname
}

##############################################################################
#
#    Description: Checks if ios_config command was successful, if not 
#                 returns error
#                 
#    IN: args will be the config command 
#    OUT:Gives error message if the configuration on the router was
#        successful otherwise prints nothing.   
#       
##############################################################################  

proc ios_conf {args} {

    set config_list [list ios_config]
    foreach arg $args {
        lappend config_list $arg
    }

    if {[catch {[eval $config_list]} err]} {
        if {[regexp {does not exist} $err] == 1} {
            set err [string trim $err]
            puts "\n\nERROR is \"$err\""        
            return 1
        } elseif {[regexp {not erased} $err] == 1} {
            set err [string trim $err]
            puts "\n\nERROR is \"$err\""
            return 1
        } elseif {[regexp {Erasing policy:} $err] == 1} {   
            return 0    
        } else {
            set err [string trim $err]
            puts "\n\nerr is \"$err\" --- Failed to configure the \
                  command \"$config_list\""   
            return 1     
        }
    }

}

#############################################################################
#
#    Description: Executes show run command and parses the output for
#                 configuration corresponding to the input entered by the
#                 user and returns it  
#
#    IN: The unique keyword to look for in the show run output   
#    OUT: Returns a list of configurations on the router corresponding to
#         the keyword entered 
#  
#############################################################################

proc exec_show_run {keyword} {
   
    global show_run_output
    set output ""
    
    if {[catch {exec "show event man env"} show_run_output] != 0} {
        puts stderr "'show run' did not execute"
    } else {
        foreach line [split $show_run_output "\n"] {
            switch -regexp -- $keyword {
                cpu {
                    if {[regexp {_svcdiag_cpu} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                memory {
                    if {[regexp {_svcdiag_mem} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                buffer {
                    if {[regexp {_svcdiag_buf} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                default {
                    puts "no input"
                }
            }    
            ### end of switch ###
        }
        ### end of foreach ### 
        
        if {[llength $output] != 0} { 
            return $output
        } else {
             puts stderr "\n\nSERVICE DIAGNOSTICS ERROR:\n\n$keyword resource monitoring was not deployed"
            return 1
        }
    }
}

#############################################################################
#
#    Description: Unconfigures the list of configuration that is passed 
#                 to it as input
#
#    IN: The list of configuration commands to be unconfigured   
#    OUT: NONE 
#  
#############################################################################

proc unconfigure {output} {

    set i 0
    
    while {$i < [llength $output]} {
        set var [lindex $output $i]
        ios_config "no event manager environment [lindex $var 1] \
        [lindex $var 2]"
        set i [expr $i + 1]
    }
}

##############################################################################
#
#    Description: Removes the policy if only one resource is configured
#                 and returns the value of 0 or 1 
#
#    IN: Takes the policy name and process name as input 
#    OUT: Returns 1 if the unconfiguration of the policy is successful
#         otherwise returns 0
#
##############################################################################

proc unconfig_erm {pol_name proc_name} {

    set err 0 
    if {$proc_name == "interrupt"} {
        if {[ios_conf "resource policy" "no user global $pol_name"\
           "no policy $pol_name global" "end"] == "1"} {
            set err 1 
        } else {
            set err 0
        }
    } else {
        if {[ios_conf "resource policy" "no user \"$proc_name\" iosprocess\
            $pol_name" "no policy $pol_name type iosprocess" "end"] == "1"} {
            set err 1
        } else {
            set err 0   
        }
    }
    return $err

}

##############################################################################
#
#    Description: Takes care of removing the resource configuration for a 
#                 policy when there are 2 or 3 resources configured for a 
#                 policy
#
#    IN: Takes the policy configuration, resource type and process name
#        as input 
#    OUT: Returns the value "removed" if the configuration for a particular
#         resource is removed, "not removed" if the configuration is not
#         removed and "not configured" if the configuration for the 
#         resource is missing
#
##############################################################################

proc remove_res_details {res_config res_type process} {

    set err "not removed"
    set pol_config ""

    switch [llength $res_config] {

        10 -
        11 {
    # when 2 resources are configured
               set res1 [split [lindex $res_config 2] " "] 
               set res2 [split [lindex $res_config 5] " "]
               if {[lindex $res1 4] == "$res_type"} {
                   if {[ios_conf "resource policy" "[lindex $res_config 0]" \
                   "[lindex $res_config 1]" "[lindex $res_config 2]" \
                   "no [lindex $res_config 3]" "end"] != "1" } {
                       puts "The $res_type configuration for the process \
                       $process is removed" 
                       set err "removed"
                   } else {
                       puts "The $res_type configuration for the process \
                       $process is not removed"
                       set err "not removed"
                   }
               } elseif {[lindex $res2 4] == "$res_type"} {
                    if {[ios_conf "resource policy" "[lindex $res_config 0]" \
                    "[lindex $res_config 1]" "[lindex $res_config 5]" \
                    "no [lindex $res_config 6]" "end"] != "1"} {
                        puts "The $res_type configuration for the process \
                        $process is removed"
                        set err "removed"
                    } else {
                        puts "The $res_type configuration for the process \
                        $process is not removed"
                        set err "not removed"
                    }
               } else {
                   puts "\n\nSERVICE DIAGNOSTICS ERROR:"
                   puts "\nThere are no $res_type inputs configured for\
                   this process $process"
                   set err "not configured"
               }
           }
        14 { 
    # when 3 resources are configured
               set res1 [split [lindex $res_config 2] " "]
               set res2 [split [lindex $res_config 5] " "]
               set res3 [split [lindex $res_config 8] " "]
               if {[lindex $res1 4] == "$res_type"} {
                   if {[ios_conf "resource policy" "[lindex $res_config 0]" \
                   "[lindex $res_config 1]" "[lindex $res_config 2]" \
                   "no [lindex $res_config 3]" "end"] != "1" } {
                       puts "The $res_type configuration for the process \
                       $process is removed"
                       set err "removed"
                   } else {
                       puts "The $res_type configuration for the process \
                       $process is not removed"
                       set err "not removed"
                   }
               } elseif {[lindex $res2 4] == "$res_type"} {
                    if {[ios_conf "resource policy" "[lindex $res_config 0]" \
                    "[lindex $res_config 1]" "[lindex $res_config 5]" \
                    "no [lindex $res_config 6]" "end"] != "1"} {
                        puts "The $res_type configuration for the process \
                        $process is removed"
                        set err "removed"
                    } else {
                        puts "The $res_type configuration for the process \
                        $process is not removed"
                        set err "not removed"
                    }
               } elseif {[lindex $res3 4] == "$res_type"} {
                    if {[ios_conf "resource policy" "[lindex $res_config 0]" \
                    "[lindex $res_config 1]" "[lindex $res_config 8]" \
                    "no [lindex $res_config 9]" "end"] != "1"} {
                        puts "The $res_type configuration for the process \
                        $process is removed"
                        set err "removed"
                    } else {
                        puts "The $res_type configuration for the process \
                        $process is not removed"
                        set err "not removed"
                    }
               } else {
                   puts "\n\nSERVICE DIAGNOSTICS ERROR:"
                   puts "\nThere are no $res_type inputs configured for\
                   this process $process"
                   set err "not configured"  
               }
           }
           default {
               set err "not removed"
           }
    }
    return $err
} 

##############################################################################
#
#    Description: Checks if the configuration for a policy exists and
#                 returns appropriate values based on the existence
#
#    IN: Takes process name and resource type(cpu, memory or buffer) as 
#        input 
#    OUT: Returns a value of "1" if the policy configuration is missing 
#         otherwise returns the output of the policy configuration 
#
##############################################################################

proc check_pol_exists {pname res_type} {

    set name [getProcessName $pname]

    if {$name == "intr"} {
        set pol_detail [exec "sh run | section i policy sdiag-interrupt"]
    } else {
        set pol_detail [exec "sh run | section i policy sdiag-$name"]
    }
  
    if {$pol_detail == ""} {
        puts "\n\nSERVICE DIAGNOSTICS ERROR:"
        puts "\nThis process $pname was not registered for threshold \
              crossing alert" 
        return 1 
    } else {
        return $pol_detail
    } 
}

##############################################################################
#
#   Description: Removes the resource policy details for a particular 
#                process by examining the ERM configuration for each
#                of the policy
#
#   IN: "pol_detail" which is the detail of a particular policy, "res_type"
#       which is either 'cpu' or 'memory' or 'buffer' and "pname" which
#       is the process name
#   OUT: Returns error message "removed", "not removed" or "" based on the
#        return  value of procedures called within this procedure       
#                
##############################################################################

proc remove_res {pol_detail res_type pname} { 
 
    set err ""
    set name [getProcessName $pname]
    set output [split $pol_detail "\n"]
        foreach line $output {
            lappend pol $line 
        }
        if {[llength $output] > 8} {
            set err [remove_res_details $pol $res_type $pname]
        } else {
            set res [split [lindex $output 2] " "]
            if {[lindex $res 4] == $res_type} {
                if {[unconfig_erm sdiag-$name $pname] == "1"} {
                    puts "\nERM policy sdiag-$name was not removed"
                    set err "not removed"   
                } else {
                    if {[unreg_eem_policy sdiag-$name.tcl] == ""} {
                        set user_pol_dir [exec show event man dir user pol]
                        if {[regexp "not configured" $user_pol_dir match]} {
                             puts "Cannot determine user policy directory for \
                             cleanup"
                        } else {
                             if {[catch { \
                             file delete [file join $user_pol_dir \
                             sdiag-$name.tcl]} del_err ]} {
                                  puts "Cannot delete policy file - $del_err"
                              }
                        }
                        puts "\nERM policy sdiag-$name was removed \
                             successfully"
                        set err "removed"
                    }
                }
            } else {
                puts "The resource $res_type is not configured for this \
                      process $pname"
                      set err "not removed"
            }
        }
        return $err 
}

### Script Execution starts here ###

set error "\nFOR CPU"
append error "\nUsage:tclsh <disk#:>/<dir_name>/\
             <sdiag_resource_undeploy.tcl> <cpu>\
             <process name>"
append error "\n"
append error "\nFOR MEMORY"
append error "\nUsage:tclsh <disk#:>/<dir_name>/\
             <sdiag_resource_undeploy.tcl> <memory>\
              <process name>" 
append error "\n"
append error "\nFOR BUFFER"
append error "\nUsage:tclsh <disk#:>/<dir_name>/\
              <sdiag_resource_undeploy.tcl> <buffer>\
                            <process name>"

if {$argc == 0} {
    if {$tcl_interactive == "1"} {
    } else {
        puts "\n\nSERVICE-DIAGNOSTICS ERROR: No inputs were given"
        puts $error
        return
    }
} else {
    if {$argc != 2} {
        puts "\n\nSERVICE-DIAGNOSTICS ERROR: Wrong number of arguments"
        puts $error
        return
    }
}

if {$argc == 2} {
    set type [lindex $argv 0]
    set proc_name [lindex $argv 1]
}

switch $type {
    
    cpu {

        set res_pol_config [exec "sh run | section i resource policy"]
        set result [split $res_pol_config "\n"]
        set count 0

        foreach line $result {
            if {[regexp {cpu} $line] == 1} {
                set count [expr $count + 1]
            }
        }

        set policy [check_pol_exists $proc_name cpu] 
        if {$policy == 1} {
            return
        } else {
            set pol_detail $policy
            if {$proc_name == "intr"} {
                set pol_removal_status [remove_res $pol_detail cpu interrupt]
            } else {
                set pol_removal_status [remove_res $pol_detail cpu $proc_name] 
            }
        }    

        if {$count == 1} {
            if {$pol_removal_status == "removed"} {
                set output [exec_show_run cpu]
                if {$output != "1"} {
                    unconfigure $output
                } else return
            }
        }
    }
    
    memory {
  
        set res_pol_config [exec "sh run | section i resource policy"]
        set result [split $res_pol_config "\n"]
        set count 0

        foreach line $result {
            if {[regexp {memory} $line a] == 1} {
                set count [expr $count + 1]
            }
        } 

        set policy [check_pol_exists $proc_name memory]
        if {$policy == 1} {
            return
        } else {
            set pol_detail $policy
            set pol_removal_status [remove_res $pol_detail memory $proc_name]
        }

        if {$count == 1} {
            if {$pol_removal_status == "removed"} {
                set output [exec_show_run memory]
                if {$output != "1"} {
                    unconfigure $output
                } else return
            }
        }
    }
 
    buffer {
    
        set res_pol_config [exec "sh run | section i resource policy"]
        set result [split $res_pol_config "\n"]
        set count 0

        foreach line $result {
            if {[regexp {buffer} $line] == 1} {
                set count [expr $count + 1]
            }
        }

        set policy [check_pol_exists $proc_name buffer]
        if {$policy == 1} {
            return
        } else {
            set pol_detail $policy
            set pol_removal_status [remove_res $pol_detail buffer $proc_name]
        }
 
        if {$count == 1} {
            if {$pol_removal_status == "removed"} {
                set output [exec_show_run buffer]
                if {$output != "1"} {
                    unconfigure $output
                } else return
            }
        }
  
    } 

    default {
        global error
        puts "\nSERVICE DIAGNOSTICS ERROR: Correct input was not entered"
        puts $error
    }
} 



#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3038303430373230323032345a302306092a864886f70d010904
#311604143e692abefce68d17804ba6b8a68e943cd6a7cbe0305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201003008ab86ca9ba6f3f3cd086b8b7dcba2a1651333f04093
#1986d379ff35321e440d09000fcc0d5b3870e42881e2e112a174a5a7b304
#7cfa635a93862663a0a86b93ff8b3eae4777c1bfbeda3630ed44e1672b4a
#83a04592e800867628facdc90d2ecd4efcb91a54e8dc6bae3c14fb71c056
#4027218895e9d4648417867b013b697e18aafc3bdace0654d1bb7d958582
#44ea5756656350909cc52650fe0de61d038136d6f4812035f86733c2b21a
#7da7cbb1e3b326128214a6335e40e7f7681e9f36cbcb75d85e32b76aba03
#48b7bb1f23cce9725a7ee409f81c3dfa142ffcd1ff3c00ad9103c24006b8
#545a4b109d6fe000f36eba31c1436d5987884cdebc87ab
